
class FunctionParser
{
	String fn_original="sin(x)";
	private char function[];
	private boolean valid;
	private int i;

	public FunctionParser()
	{
		function=new char[50];
		valid=true;
	}

	private String FUNCTIONS[]=
	{
		"sin","cos","tan",
		"asin","acos","atan",
		"ln","log","exp",
		"abs",
		"sinh","cosh","tanh"
	};

	private char FUNCTIONCODES[]=
	{
		's','c','t',
		'S','C','T',
		'l','L','e',
		'a',
		'p','q','r'
	};

	private String CONSTANTS[]=
	{
		"PI","E"
	};

	private char CONSTANTCODES[]=
	{
		'P','E'
	};

	private double CONSTANTVALUES[]=
	{
		3.14159,2.71828
	};

	// Functions To Check State Of Character //
	boolean isOperand(char c)
	{
		if((c>='0' && c<='9') || c=='.' || c=='x'
				|| c=='+' || c=='-'
				|| c=='P' || c=='E' )
			return true;
		else
			return false;
	}

	boolean isSign(char c)
	{
		if(c=='+' || c=='-')
			return true;
		else
			return false;
	}

	boolean isOperator(char c)
	{
		if(c=='+' || c=='-' || c=='*' || c=='/' || c=='^'
			|| c=='(' || c==')' )
			return true;
		else
			return false;
	}

	// To Check If Function Is Used //
	boolean isFunction(String str,int k)
	{
		for(int j=0;j<FUNCTIONS.length;j++)
		{
			if(str.substring(k).startsWith(FUNCTIONS[j]))
			{
				return true;
			}
		}

		return false;
	}

	// To Retrieve Function //
	char getFunction(String str)
	{
		// Retrieve String //
		for(int j=0;j<FUNCTIONS.length;j++)
		{	
			if(str.substring(i).startsWith(FUNCTIONS[j]))
			{
				i+=FUNCTIONS[j].length();
				return FUNCTIONCODES[j];
			}
		}

		return ' ';
	}

	// To Check If It Is Function Code //
	boolean isFunctionCode(char c)
	{
		for(int k=0;k<FUNCTIONCODES.length;k++)
		{
			if(c==FUNCTIONCODES[k])
				return true;
		}
		return false;
	}

	// To Check If Constant //
	boolean isConstant(String str,int k)
	{
		for(int j=0;j<CONSTANTS.length;j++)
		{
			if(str.substring(k).startsWith(CONSTANTS[j]))
			{
				return true;
			}
		}
		return false;
	}

	// To Retrieve Constant Code //
	char getConstant(String str)
	{
		for(int j=0;j<CONSTANTS.length;j++)
		{
			if(str.substring(i).startsWith(CONSTANTS[j]))
			{
				i+=CONSTANTS[j].length()-1;
				return CONSTANTCODES[j];
			}
		}

		return ' ';
	}

	// To Check If It Is Constant Code //
	boolean isConstantCode(char c)
	{
		for(int k=0;k<CONSTANTS.length;k++)
		{
			if(c==CONSTANTCODES[k])
				return true;
		}
		return false;
	}

	// To Get Constant Value //
	double getConstantValue(char c)
	{
		for(int k=0;k<CONSTANTS.length;k++)
		{
			if(c==CONSTANTCODES[k])
				return CONSTANTVALUES[k];
		}
		return 0;
	}


	// Precedence Functions //
	/*
	a -> Operator At Top Of Stack
	b -> Our Read Operator
	return (a>b)
	*/

	boolean isPreceding(char a,char b)
	{
		if(b=='(')
			return false;
		else
		if(a=='(' && b==')')
			return false;
		else
		if(a=='(')
			return false;
		else
		if(b==')')
			return true;

		for(int k=0;k<FUNCTIONCODES.length;k++)
		{
			if(a==FUNCTIONCODES[k])
				return false;
		}

		switch(a)
		{
			case '+':
			{
				switch(b)
				{
					case '+':
						return true;
					case '-':
						return true;
					case '*':
						return false;
					case '/':
						return false;
					case '^':
						return false;
				}
			}
			case '-':
				switch(b)
				{
					case '+':
						return true;
					case '-':
						return true;
					case '*':
						return false;
					case '/':
						return false;
					case '^':
						return false;
				}
			case '*':
				switch(b)
				{
					case '+':
						return true;
					case '-':
						return true;
					case '*':
						return true;
					case '/':
						return true;
					case '^':
						return false;
				}
			case '/':
				switch(b)
				{
					case '+':
						return true;
					case '-':
						return true;
					case '*':
						return true;
					case '/':
						return true;
					case '^':
						return false;
				}
			case '^':
				switch(b)
				{
					case '+':
						return true;
					case '-':
						return true;
					case '*':
						return true;
					case '/':
						return true;
					case '^':
						return true;
				}
			default:
				return false;
		}
	}

	// To Evaluate The PostFix Expression For Given Value //
	public double calculate(double x)
	{
		if(!valid)
			setFunction(fn_original);

		MyStack values=new MyStack();
		double temp=0;

		for(int i=0;function[i]!='\0';i++)
		{
			// Check If Operator //
			if(isOperator(function[i]) && (function[i+1]==' ' || function[i+1]=='\0'))
			{
				switch(function[i])
				{
					case '+':
						temp=values.pop()+values.pop();
						break;
					case '-':
						temp=values.pop();
						temp=values.pop()-temp;
						break;
					case '*':
						temp=values.pop()*values.pop();
						break;
					case '/':
						temp=values.pop();
						if(temp==0)
							temp=0.001;
						temp=values.pop()/temp;
						break;
					case '^':
						temp=values.pop();
						temp=Math.pow(values.pop(),temp);
						break;
					default:
						break;
				}
				values.push(temp);
			}
			else
			// Check If Operand //
			if(isOperand(function[i]))
			{
				int sign=1;

				// If First Char Is Sign Get It //
				if(isSign(function[i]))
				{
					switch(function[i])
					{
						case '+':
							sign=+1;
						case '-':
							sign=-1;
					}
					i++;
				}

				if(function[i]=='x')
				{
					temp=sign*x;
				}
				else
				if(isConstantCode(function[i]))
				{
					temp=sign*getConstantValue(function[i]);
				}
				else
				{
					int j=0;
					char sub[]=new char[10];

					while(function[i+j]!=' ')
					{
						sub[j]=function[i+j];
						j++;
					}

					try
					{
						String tempstr=new String(sub,0,j);
						temp=sign*Double.valueOf(tempstr).doubleValue();
					}
					catch(NumberFormatException e)
					{
						temp=1;
					}
					
					i+=j-1;
				}
				values.push(temp);
			}
			else
			// Check If Code //
			if(isFunctionCode(function[i]))
			{
				switch(function[i])
				{
					case 's':
						temp=Math.sin(values.pop());
						break;
					case 'c':
						temp=Math.cos(values.pop());
						break;
					case 't':
						temp=Math.tan(values.pop());
						break;
					case 'S':
						temp=Math.asin(values.pop());
						break;
					case 'C':
						temp=Math.acos(values.pop());
						break;
					case 'T':
						temp=Math.atan(values.pop());
						break;
					case 'l':
						temp=Math.log(values.pop());
						break;
					case 'L':
						temp=Math.log(values.pop())/2.302585;
						break;
					case 'e':
						temp=Math.exp(values.pop());
						break;
					case 'a':
						temp=Math.abs(values.pop());
						break;
					case 'p':
						temp=values.pop();
						temp=(Math.exp(temp)-Math.exp(-temp))/2.0;
						break;
					case 'q':
						temp=values.pop();
						temp=(Math.exp(temp)+Math.exp(-temp))/2.0;
						break;
					case 'r':
						temp=values.pop();
						temp=(Math.exp(temp)-Math.exp(-temp))/(Math.exp(temp)+Math.exp(-temp));
						break;
					default:
						break;
				}
				values.push(temp);
			}
			else
			if(function[i]==' ')
			{
				continue;
			}
		}

		return values.pop();
	}

	public boolean setFunction(String str)
	{
		// Analyze Equation //
		CharStack opStack=new CharStack();
		
		char eqn[]=str.toCharArray();
		char c;
		
		int num=0;
		int rank=0;
		boolean opActive=false;	// To Allow Signs To Variables //

		for(i=0;i<eqn.length;i++)
		{
			c=eqn[i];

			// Check If Function //
			if(isFunction(new String(eqn),i))
			{
				char tempchar=getFunction(new String(eqn));
				opStack.push(tempchar);
			}
			else
			// Check If Operator //
			if(isOperator(c) && (c=='(' || opActive))
			{
				while(!opStack.isEmpty() && isPreceding(opStack.getTop(),c))
				{
					function[num++]=' ';
					function[num++]=opStack.pop();
				}

				if(c!=')')
					opStack.push(c);
				else
					opStack.pop();

				if(c!='(' && c!=')')
					rank--;

				if(c!=')')
					opActive=false;
			}
			else
			// Check If Operand //
			if(isOperand(c))
			{
				if(isConstant(new String(eqn),i))
				{
					function[num++]=getConstant(new String(eqn));
				}
				else
				{
					int j=0;

					do
					{
						function[num++]=c;
						j++;

						if(i+j>=eqn.length)
							break;

						c=eqn[i+j];
					}
					while(c!='\0' && isOperand(c) && c!='+' && c!='-');

					i+=j-1;
				}
				rank++;
				opActive=true;
			}
			else
			// Error //
			{
				return (valid=false);
			}
			function[num++]=' ';
		}

		if(rank!=1)
		{
			return (valid=false);
		}

		while(!opStack.isEmpty())
		{
			function[num++]=' ';
			function[num++]=opStack.pop();
		}

		function[num]='\0';
		fn_original=str;

		return (valid=true);
	}

	public String getFunction()
	{
		return fn_original;
	}
}