
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class GraphApplet extends Applet implements ActionListener
{
	final double XSCALE=0.85;
	final double YSCALE=0.85;
	final String msgString="By Dattaraj J. Rao ( jagrao@goa1.dot.net.in )";
	final String errString="Error In Function";

	int w,h;

	Grapher grapher;
	FunctionParser function;

	double XMIN,XMAX;
	int POINTS;

	TextField txt_fn,txt_xmin,txt_xmax,txt_points;
	Button btn_draw,btn_set,btn_grid,btn_view;
	Label lbl_msg;

	public void init() 
	{
		w=getSize().width;
		h=getSize().height;

		grapher=new Grapher();
		function=new FunctionParser();
		function.setFunction("sin(x)");

		setLayout(null);
		setBackground(new Color(225,175,175));

		grapher.setBounds(10,10,(int)(w*XSCALE)-20,(int)(h*YSCALE)-20);
		add(grapher);
		grapher.init();
		
		// Controls Panel //
		Panel controls=new Panel();
		controls.setLayout(new GridLayout(12,1,0,5));
		
		controls.add(new Label(" X ( Min ) :"));
		txt_xmin=new TextField();
		controls.add(txt_xmin);

		controls.add(new Label(" X ( Max ) :"));
		txt_xmax=new TextField();
		controls.add(txt_xmax);

		controls.add(new Label(" Points :"));
		txt_points=new TextField();
		controls.add(txt_points);

		btn_set=new Button("Set");
		btn_set.addActionListener(this);
		controls.add(btn_set);

		controls.add(new Label(""));

		btn_grid=new Button("Grid ( On/Off)");
		btn_grid.addActionListener(this);
		controls.add(btn_grid);

		btn_view=new Button("Change View");
		btn_view.addActionListener(this);
		controls.add(btn_view);

		controls.add(new Label(""));
		controls.add(new Label(""));

		add(controls);
		controls.setBounds((int)(w*XSCALE),0,(int)(w*(1-XSCALE))-10,(int)(h*YSCALE));

		// Function Panel //
		Panel fnPanel=new Panel();
		fnPanel.setLayout(new GridLayout(2,1,0,10));

		// Top Panel Of Function Panel //
		Panel fn_top=new Panel();
		fn_top.setLayout(new GridLayout(1,3,5,0));
		
		fn_top.add(new Label(" Enter The Function : "));
		
		txt_fn=new TextField();
		fn_top.add(txt_fn);
		txt_fn.setText(function.getFunction());
		
		btn_draw=new Button("Draw Graph");
		btn_draw.addActionListener(this);
		fn_top.add(btn_draw);

		fnPanel.add(fn_top);
		
		// Bottom Of Function Panel ( Error Text ) //
		lbl_msg=new Label(msgString,Label.CENTER);
		fnPanel.add(lbl_msg);
		
		add(fnPanel);
		fnPanel.setBounds(0,(int)(h*YSCALE),w,(int)(h*(1-YSCALE)));

		XMIN=-10.0;
		XMAX=10.0;
		POINTS=50;
		function.setFunction("sin(x)");
		setData();
	}

	public void paint(Graphics g)
	{
	}

	private void drawGraph()
	{
		grapher.setXScale((float)XMIN,(float)XMAX,POINTS);
		
		double temp;
		double XSTEP=(XMAX-XMIN)/(POINTS-1);

		for(double x=XMIN;x<XMAX+(XMAX-XMIN)/POINTS;x+=XSTEP)
		{
			grapher.addPoint((float)x,(float)function.calculate(x));
		}
		grapher.repaint();
	}

	private void retrieveData()
	{
		txt_xmin.setText(XMIN+"");
		txt_xmax.setText(XMAX+"");
		txt_points.setText(POINTS+"");
		txt_fn.setText(function.getFunction());
	}

	private void setData()
	{
		if(!function.setFunction(txt_fn.getText()))
		{
			lbl_msg.setText(errString);
			txt_fn.requestFocus();
			return;
		}
		else
		{
			lbl_msg.setText(msgString);
		}

		try
		{
			XMIN=Double.valueOf(txt_xmin.getText()).doubleValue();
		}
		catch(NumberFormatException exp)
		{
		}

		try
		{
			XMAX=Double.valueOf(txt_xmax.getText()).doubleValue();
		}
		catch(NumberFormatException exp)
		{
		}
		
		try
		{
			POINTS=Integer.parseInt(txt_points.getText());
		}
		catch(NumberFormatException exp)
		{
		}
		
		retrieveData();
		drawGraph();
		txt_fn.requestFocus();
	}

	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource()==btn_draw || e.getSource()==btn_set)
			setData();
		else
		if(e.getSource()==btn_grid)
			grapher.changeGrid();
		else
		if(e.getSource()==btn_view)
			grapher.changeView();
	}

}